function [P_receiver,tESSIEinitial,tESSIEmatrix,tESSIEmatrixsubtimes,tESSIEsourceterm,tESSIEedgesignals,tESSIEpropagate] = ESIE0integralequation(Filepath,Filestem,planedata,edgedata,edgetoedgedata,...
    Hsubmatrix,listofsubmatrices,edgepairlist,edgetripletlist,submatrixcounter,nedgeelems,...
    bigmatrixstartnums,bigmatrixendnums,reftoshortlist,isthinplanetriplet,isthinplaneedgepair,...
        quadraturematrix_pos,quadraturematrix_weights,Sdata,doaddsources,...
        Rdata,inteq_ngauss,inteq_discretizationtype,frequencies,...
        inteq_solmethod,inteq_niter,inteq_savesourcesignals,Rstart)
% ESIE0integralequation calculates the sound pressure representing second-
% and higher-order diffraction.
%
% Input parameters: long list
%
% Output parameters
%   P_receiver      Matrix with the diffracted pressure
%                   If nsources = 1, or all sources should be added, then
%                       the size is [nfrequencies,nreceivers]
%                   If nsources > 1, and all sources should be computed
%                       separately, then the size is
%                       [nfrequencies,nreceivers,nsources]
%   tESSIEinitial    Time required for the freq.independent part
%   tESSIEmatrix     Vector, [nfrequencies,1], with the timings for setting
%                   up the transfer matrices
%   tESSIEmatrixsubtimes        Matrix, [nfrequencie5,6], with the timings
%                   for various parts of the tESSIEmatrix
%   tESSIEsourceterm Matrix with the timings for computing the source terms
%                   If nsources = 1, or all sources should be added, then
%                       the size is [nfrequencies,1]
%                   If nsources > 1, and all sources should be computed
%                       separately, then the size is
%                       [nfrequencies,nsources]
%   tESSIEedgesignals    Matrix with the timings for computing the
%                       edge source signals
%                   If nsources = 1, or all sources should be added, then
%                       the size is [nfrequencies,1]
%                   If nsources > 1, and all sources should be computed
%                       separately, then the size is
%                       [nfrequencies,nsources]
%   tESSIEpropagate  Matrix with the timings for computing the propagated
%                   field
%                   If nsources = 1, or all sources should be added, then
%                       the size is [nfrequencies,nreceivers]
%                   If nsources > 1, and all sources should be computed
%                       separately, then the size is
%                       [nfrequencies,nreceivers,nsources]
%
% Uses functions ESIE0distelements, ESIE0calcedgeinteqmatrixsub2_mex, ESIE0integ_souterm, ESIE0calcpropagatematrix
%           
% Peter Svensson (svensson@iet.ntnu.no)  8 April 2015  
%                       
% [P_receiver,tESSIEinitial,tESSIEmatrix,tESSIEmatrixsubtimes,tESSIEsourceterm,tESSIEedgesignals,tESSIEpropagate] = ESIE0integralequation(Filepath,Filestem,...
%         planedata,edgedata,edgetoedgedata,Hsubmatrix,listofsubmatrices,edgepairlist,edgetripletlist,submatrixcounter,nedgeelems,bigmatrixstartnums,bigmatrixendnums,reftoshortlist,isthinplanetriplet,isthinplaneedgepair,...
%         quadraturematrix_pos,quadraturematrix_weights,sdatafile,doaddsources,rdatafile,inteq_ngauss,inteq_discretizationtype,frequencies,...
%         inteq_solmethod,inteq_niter,inteq_savesourcesignals,Rstart);

%
% 31 March 2015 Introduced detailed timing, also as output parameter.
% 8 April 2015 Substantial speeding up by saving Hsubdata instead of Hsub
%              and using the accumarray function.
% 28 June 2015 Added the option of saving the edge source signals.

global CAIR SHOWTEXT JJ JJnumbofchars

if inteq_solmethod ~= 5
    error('ERROR: Only inteq_solmethod == 5 supported here');
end

nfrequencies = length(frequencies);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run an outer loop over all frequencies 
    
t00 = clock;

% eval(['load ',sdatafile])
nsources = size(Sdata.sources,1);
% eval(['load ',rdatafile])
nreceivers = size(Rdata.receivers,1);

isthinhole = 0;
if SHOWTEXT >= 3
    disp('WARNING!! isthinhole has been forced to 0')
end

% Find if there are image sources that could see any
% edges
nplanes = size(planedata.planecorners,1);
nedges = size(edgedata.edgecorners,1);
vispartedgesfromIS = sparse(zeros(nplanes,nedges));

% if SHOWTEXT >= 1,
%     disp(' ')
%    disp(['      INTEGRAL EQ.: Calculating source terms '])
% end
[gaussvec1,gaussvec2] = ESIE0distelements(inteq_ngauss,inteq_discretizationtype);
gaussvectors = [gaussvec1 gaussvec2];

tESSIEinitial = etime(clock,t00);

tESSIEmatrix = zeros(nfrequencies,1);
tESSIEmatrixsubtimes = zeros(nfrequencies,size(listofsubmatrices,1)+1);

if nsources == 1 | doaddsources == 1
   tESSIEsourceterm = zeros(nfrequencies,1); 
else
   tESSIEsourceterm = zeros(nfrequencies,nsources);     
end
if nsources == 1 | doaddsources == 1
   tESSIEedgesignals = zeros(nfrequencies,1); 
else
   tESSIEedgesignals = zeros(nfrequencies,nsources);     
end
if nsources == 1 | doaddsources == 1
   tESSIEpropagate = zeros(nfrequencies,nreceivers); 
else
   tESSIEpropagate = zeros(nfrequencies,nreceivers,nsources);     
end
tESSIEmatrixsubtimes = zeros(nfrequencies,10);

for ifreq = 1:nfrequencies                  
    t00 = clock;
    frequency = frequencies(ifreq);
    if SHOWTEXT >= 1,
        disp(['   Frequency: ',num2str(frequency),' Hz'])
    end

    % ----------------------------------------------------------------------------------------------
    % Create the big H-matrix, possibly in the form of separate sub-matrices

    nbig = bigmatrixendnums(end);

    if inteq_solmethod <= 3
        HBIG = zeros(nbig,nbig);
    end
    refto = edgetoedgedata.reftoshortlistE;

    k = 2*pi*frequency/CAIR;

    if SHOWTEXT >= 2
        disp(' ')
        disp(['      Building the H matrix of size (',int2str(nbig),',',int2str(nbig),')'])
        disp(['      with ',int2str(submatrixcounter),' submatrices'])
    end

    alltripletsthin = prod(double(isthinplanetriplet));
    notripletsthin = prod(1-double(isthinplanetriplet));

    % In the for-loop, nn will be the Hsub number
    % The ii value will refer to the row-number in the edgepairlist for the 
    % receiving edge-pair.
    % The jj value will refer to the row-number in the edgepairlist for the 
    % sending edge-pair.

    n1prev = 0;
    n2prev = 0;
    n3prev = 0;
    shortlistprev = 0;

    Hsub = [];

    if isempty(reftoshortlist)
        symmetrycompression = 0;
    else    
        symmetrycompression = 1;
    end

    if symmetrycompression < 1
       reftoshortlist = 1:size(listofsubmatrices,1);
    end

    inteqversion = 'new';
    nrowspertriplet = prod(nedgeelems(edgetripletlist).').';
    rowend_bigdatamatrix = cumsum(nrowspertriplet);
    rowstart_bigdatamatrix = [1;rowend_bigdatamatrix(1:end-1)+1];
    nrows = rowend_bigdatamatrix(end);

%     bigdatamatrix1 = zeros(nrows,4);
%     bigdatamatrix2 = zeros(nrows,1);        

    Hsubconstant = -1/4/pi/2;

    tESSIEmatrixsubtimes(ifreq,1) = etime(clock,t00);
    
    for nn = 1:size(listofsubmatrices,1)

        if nn == 1
            t00sub = clock;
        end
        ii = listofsubmatrices(nn,2);
        jj = listofsubmatrices(nn,3);

        edge1 = edgetripletlist(nn,3);
        n1 = nedgeelems(edge1);

        edge2 = edgetripletlist(nn,2);                   
        n2 = nedgeelems(edge2);
           len2 = edgedata.edgelengthvec(edge2);

        edge3 = edgetripletlist(nn,1);
        n3 = nedgeelems(edge3);
    %     disp(['triplet no. ',int2str(nn),': n1,n2,n3 = ',int2str(n1),',',int2str(n2),',',int2str(n3)])

        shortlistnumber = reftoshortlist(nn);

        if alltripletsthin == 1
            thinplaneboost = 2;
        else
            if notripletsthin == 1
                thinplaneboost = 1;
            else
                thinplaneboost = isthinplanetriplet(nn)+1;
            end
        end

        ind_start = 1 + (bigmatrixstartnums(ii)-1) + (bigmatrixstartnums(jj)-1)*nbig;

       if SHOWTEXT >= 3
           disp(' ')
            disp(['      H-submatrix ',int2str(nn),': From edge ',int2str(edge1),' via edge ',int2str(edge2),' to edge ',int2str(edge3)])
            disp(['      of size ',int2str(n3*n2),' by ',int2str(n2*n1),'. Thinplaneboost = ',int2str(thinplaneboost)])
            if symmetrycompression ==1
                disp(['      stored as shortlist number ',int2str(shortlistnumber)])
            end
       end

       % Build vertical [n3,n2] matrices and expand them horizontally
       % NB! n1 refers to edge 1, which is the source edge
       %     n3 refers to edge 3, which is the target edge
       % edgetriplets written in order [edge3 edge2 edge 1]

       if n1 ~= n1prev || n2 ~= n2prev || n3 ~= n3prev

% % % % %            n3vertmat = uint8(1:n3);
% % % % %            n3vertmat = reshape(repmat(n3vertmat,n2,1),n2*n3,1);           
% % % % %            n3vertmat = n3vertmat(:,ones(1,n1*n2));
% % % % % 
% % % % %            n2vertmat = uint8(1:n2).';
% % % % %            n2vertmat = repmat(n2vertmat,n3,1);
% % % % %            n2vertmat = n2vertmat(:,ones(1,n1*n2));
% % % % % 
% % % % %        % Build horizontal n2 and n1 matrices and expand them vertically
% % % % %            n1hormat = uint8(1:n1);
% % % % %            n1hormat = repmat(n1hormat,1,n2);
% % % % %            n1hormat = n1hormat(ones(n3*n2,1),:);
% % % % % 
% % % % %            n2hormat = uint8(1:n2);
% % % % %            n2hormat = reshape(repmat(n2hormat,n1,1),1,n2*n1);           
% % % % %            n2hormat = n2hormat(ones(n3*n2,1),:);
% % % % % 
% % % % %        % Now, we should use only the combinations where n2vertmat ==
% % % % %        % n2hormat! Otherwise we get too many combinations: there should
% % % % %        % be n1*n2*n3 unique combinations, but the Hsub-matrix has size
% % % % %        % [n3*n2,n2*n1], which has n2 too many elements!
% % % % %        %
% % % % %        % Once we have the iv, which identifies the wanted matrix
% % % % %        % elements, we can throw away n2hormat because it is identical
% % % % %        % to n2vertmat in the wanted matrix elements!
% % % % % 
% % % % %            ivuse = find(n2hormat==n2vertmat);
% % % % %            n3vertmat = n3vertmat(ivuse);
% % % % %            n2vertmat = n2vertmat(ivuse);
% % % % %            n1hormat  = n1hormat(ivuse);

           [ivuse,n1hormat,n2vertmat,n3vertmat] = ESIE0calcedgeinteqmatrixsub2_mex(n1,n2,n3);       

           n1hormat = uint8(n1hormat);
           n2vertmat = uint8(n2vertmat);
           n3vertmat = uint8(n3vertmat);
           ivuse = uint32(ivuse);
           
       end

       if nn == 1
          tESSIEmatrixsubtimes(ifreq,2) = etime(clock,t00sub);
          t00sub = clock;           
       end
       
       % Build vectors that will be referred to by the n1,n2,n3
       % matrices

       if n1 ~= n1prev
            n1vec = full(quadraturematrix_pos(n1,1:n1));
            n1vec = n1vec(:);
            weightvec = full(quadraturematrix_weights(n1,1:n1));
            weightvec = weightvec(:);
            n1prev = n1;
       end       
       dzvec = weightvec*edgedata.edgelengthvec(edge1);

        if n2 ~= n2prev
            n2vec = full(quadraturematrix_pos(n2,1:n2));
            n2vec = n2vec(:);
            n2prev = n2;
        end
        if n3 ~= n3prev        
            n3vec = full(quadraturematrix_pos(n3,1:n3));
            n3vec = n3vec(:);
            n3prev = n3;
        end

       if nn == 1
          tESSIEmatrixsubtimes(ifreq,3) = etime(clock,t00sub);
          t00sub = clock;           
       end
        
        % Bug found and fixed 22 Sep. 2014: lines below did not always detect if another
        % edge was skewed relative to the first one, because only the
        % thetae1sho was used. Necessary to check thetae1sho and thetae2sho.

       ny = pi/(2*pi-edgedata.closwedangvec(edge2));
       thetain  = edgetoedgedata.thetae1sho(refto(edge1,edge2));
       thetain_other_end_of_edge  = edgetoedgedata.thetae2sho(refto(edge1,edge2));
    %    if abs(thetain) < 1e-10 || abs(thetain-(2*pi-closwedangvec(edge2))) < 1e-10   % Old erroneous line
       if (abs(thetain) < 1e-10 || abs(thetain-(2*pi-edgedata.closwedangvec(edge2))) < 1e-10) && abs(thetain-thetain_other_end_of_edge) < 1e-10
           acrossface_in = 1;       
       else
           acrossface_in = 0;
       end
       thetaout = edgetoedgedata.thetae1sho(refto(edge3,edge2));
       thetaout_other_end_of_edge = edgetoedgedata.thetae2sho(refto(edge3,edge2));
    %   if abs(thetaout) < 1e-10 || abs(thetaout-(2*pi-closwedangvec(edge2))) < 1e-10    % Old erroneous line
       if (abs(thetaout) < 1e-10 || abs(thetaout-(2*pi-edgedata.closwedangvec(edge2))) < 1e-10) && abs(thetaout-thetaout_other_end_of_edge) < 1e-10
           acrossface_out = 1;
       else
           acrossface_out = 0;
       end

       if SHOWTEXT >= 3
           disp(['      Acrossface-out = ',int2str(acrossface_out)])
           disp(['      Acrossface-in = ',int2str(acrossface_in), ' (not used)'])
       end

       ze2_re2 = n2vec*len2;
       ze3_re2 = edgetoedgedata.ze1sho(refto(edge3,edge2)) + n3vec*( edgetoedgedata.ze2sho(refto(edge3,edge2))-edgetoedgedata.ze1sho(refto(edge3,edge2))   );
       re3_re2 = edgetoedgedata.re1sho(refto(edge3,edge2)) + n3vec*( edgetoedgedata.re2sho(refto(edge3,edge2))-edgetoedgedata.re1sho(refto(edge3,edge2))   );

       if nn == 1
          tESSIEmatrixsubtimes(ifreq,4) = etime(clock,t00sub);
          t00sub = clock;           
       end
       if shortlistnumber ~= shortlistprev      

    %        ldist = (ze3_re2(n3vertmat(ivuse)) - ze2_re2(n2vertmat(ivuse))).^2   + re3_re2(n3vertmat(ivuse)).^2;
           ldist = (ze3_re2(n3vertmat) - ze2_re2(n2vertmat)).^2   + re3_re2(n3vertmat).^2;
           ldist = sqrt(ldist);

           n1vec = n1vec(:).';

           ze1_re2 = edgetoedgedata.ze1sho(refto(edge1,edge2)) + n1vec*( edgetoedgedata.ze2sho(refto(edge1,edge2))-edgetoedgedata.ze1sho(refto(edge1,edge2))   );
           re1_re2 = edgetoedgedata.re1sho(refto(edge1,edge2)) + n1vec*( edgetoedgedata.re2sho(refto(edge1,edge2))-edgetoedgedata.re1sho(refto(edge1,edge2))   );

           if acrossface_in*acrossface_out == 0
                xedge1start = edgetoedgedata.re1sho(refto(edge1,edge2))*sin(edgetoedgedata.thetae1sho(refto(edge1,edge2)));
                xedge1end  = edgetoedgedata.re2sho(refto(edge1,edge2))*sin(edgetoedgedata.thetae2sho(refto(edge1,edge2)));
                xe1_re2 = xedge1start + n1vec.*(xedge1end-xedge1start);
                thetae1_re2 = asin(xe1_re2./re1_re2);

                xedge3start = edgetoedgedata.re1sho(refto(edge3,edge2))*sin(edgetoedgedata.thetae1sho(refto(edge3,edge2)));
                xedge3end  = edgetoedgedata.re2sho(refto(edge3,edge2))*sin(edgetoedgedata.thetae2sho(refto(edge3,edge2)));
                xe3_re2 = xedge3start + n3vec.*(xedge3end-xedge3start);
                thetae3_re2 = asin(xe3_re2./re3_re2);            
           end

           mdist = (ze1_re2(n1hormat).' - ze2_re2(n2vertmat)).^2   + (re1_re2(n1hormat).').^2;
           mdist = sqrt(mdist);

           dz_expjkmoverm = exp(-j*k*mdist)./mdist.*dzvec(n1hormat);           

           ch = ((ze2_re2(n2vertmat) - ze1_re2(n1hormat).').*(ze2_re2(n2vertmat) - ze3_re2(n3vertmat)) + mdist.*ldist)./re1_re2(n1hormat).'./re3_re2(n3vertmat);
           ch = ( real(sqrt( ch.^2-1)) + ch ).^ny;
           ch = ( ch + 1./ch)/2;

           % PS 28 March 2011: The 4 beta terms are identical!       
           if acrossface_in == 1 && acrossface_out == 1
               beta = 4*( sin(ny*(pi + thetain + thetaout  ))./(ch - cos(ny*(pi + thetain + thetaout   ))));
           else
               beta = ( sin(ny*(pi + thetae1_re2(n1hormat).' + thetae3_re2(n3vertmat)  ))./(ch - cos(ny*(pi + thetae1_re2(n1hormat).' + thetae3_re2(n3vertmat)   ))+eps*10)) + ...
                      ( sin(ny*(pi + thetae1_re2(n1hormat).' - thetae3_re2(n3vertmat)  ))./(ch - cos(ny*(pi + thetae1_re2(n1hormat).' - thetae3_re2(n3vertmat)   ))+eps*10)) + ...
                      ( sin(ny*(pi - thetae1_re2(n1hormat).' + thetae3_re2(n3vertmat)  ))./(ch - cos(ny*(pi - thetae1_re2(n1hormat).' + thetae3_re2(n3vertmat)   ))+eps*10)) + ...
                      ( sin(ny*(pi - thetae1_re2(n1hormat).' - thetae3_re2(n3vertmat)  ))./(ch - cos(ny*(pi - thetae1_re2(n1hormat).' - thetae3_re2(n3vertmat)   ))+eps*10));
           end

           if ismember(edge1,edgedata.offedges)==1 || ismember(edge2,edgedata.offedges)==1 || ismember(edge3,edgedata.offedges)==1
               beta = beta*0;
           else
    %           disp(['Edge1: ',int2str(edge1),' edge2: ',int2str(edge2),' edge3: ',int2str(edge3)])
           end
           
           if nn == 1
            tESSIEmatrixsubtimes(ifreq,5) = etime(clock,t00sub);
            t00sub = clock;           
           end

%             eval(['Hsubdata',JJ(nn,1:JJnumbofchars(nn)),' = (2-acrossface_out)*Hsubconstant*ny*beta.*dz_expjkmoverm*thinplaneboost;'])   
%             eval(['ivuse',JJ(nn,1:JJnumbofchars(nn)),' = ivuse;'])
            eval(['Hsubdata',int2str(nn),' = (2-acrossface_out)*Hsubconstant*ny*beta.*dz_expjkmoverm*thinplaneboost;'])   
            eval(['ivuse',int2str(nn),' = ivuse;'])
            if nn == 1
                tESSIEmatrixsubtimes(ifreq,6) = etime(clock,t00sub);
                t00sub = clock;                                  
            end            
            
       end

        shortlistprev = shortlistnumber;

    end

    tESSIEmatrix(ifreq) = etime(clock,t00);
        
    %-----------------------------------------------------------------
    %-----------------------------------------------------------------
    %-----------------------------------------------------------------
    % Now all the Hsub have been calculated. Loop over all sources and
    % receivers since they can all use the same Hsubs.

    if doaddsources == 1 | nsources == 1
        t00 = clock;
        if ifreq==1
            P_receiver = zeros(nfrequencies,nreceivers);
        end
        Q_firstterm = 0;
        for isou = 1:nsources,
            ISOU = JJ(isou,1:JJnumbofchars(isou));
 
                [Q_firstterm_addition,nedgeelems] = ESIE0integ_souterm(planedata.planeeqs,edgedata,edgetoedgedata,edgepairlist,nedgeelems,bigmatrixstartnums,bigmatrixendnums,...
                inteq_ngauss,inteq_discretizationtype,Sdata.sources(isou,:),...
                Sdata.vispartedgesfroms(:,isou),vispartedgesfromIS,frequency,gaussvectors,Sdata.rSsho(Sdata.reftoshortlistS(:,isou)),...
                Sdata.thetaSsho(Sdata.reftoshortlistS(:,isou)),Sdata.zSsho(Sdata.reftoshortlistS(:,isou)));

            Q_firstterm = Q_firstterm + Q_firstterm_addition;
        end
        
        tESSIEsourceterm(ifreq) = etime(clock,t00);

        t00 = clock;
        
        if SHOWTEXT >= 2
            disp('      Starting to iterate, method 5')
        end
        
        Q = Q_firstterm;
        Qfinal = Q;   
        for iiter = 1:inteq_niter
            if SHOWTEXT >= 3
               disp(['      Iteration no. ',JJ(iiter,1:JJnumbofchars(iiter)),' of ',JJ(inteq_niter,1:JJnumbofchars(inteq_niter))])
            end
            Q_ackum = Q*0;              
            shortlistprev = 0;
% % %             Hsub = [];
            Hsubdata = [];
            ivuse = [];
            for nn = 1:size(listofsubmatrices,1)
% % %                 disp(['   Hsub',int2str(nn)])
                ivoutvec = bigmatrixstartnums(listofsubmatrices(nn,2)):bigmatrixendnums(listofsubmatrices(nn,2));
                ivinvec  = bigmatrixstartnums(listofsubmatrices(nn,3)):bigmatrixendnums(listofsubmatrices(nn,3));
                shortlistnumber = reftoshortlist(nn);
                if shortlistnumber ~= shortlistprev
%                     disp('      shifting')
% % %                    eval(['Hsub = Hsub',JJ(nn,1:JJnumbofchars(nn)),';']) 
%                    eval(['Hsubdata = Hsubdata',JJ(nn,1:JJnumbofchars(nn)),';']) 
%                    eval(['ivuse = ivuse',JJ(nn,1:JJnumbofchars(nn)),';']) 
                   eval(['Hsubdata = Hsubdata',int2str(nn),';']) 
                   eval(['ivuse = ivuse',int2str(nn),';']) 
                   nHsub1 = length(ivoutvec);
                   row_Hsub = rem(ivuse-1, nHsub1) + 1;
                   col_Hsub = (ivuse-row_Hsub)/nHsub1 + 1;
                   shortlistprev = shortlistnumber;                   
%                 else
%                                     disp('      not shifting')
                end


                Q_ackum(1:ivoutvec(end)) = Q_ackum(1:ivoutvec(end)) + accumarray(ivoutvec(1)-1+row_Hsub,Hsubdata.*Q(ivinvec(1)-1+col_Hsub));                      
            end
            Q = Q_ackum; 
            Qfinal = Qfinal + Q; 
        end

        tESSIEedgesignals(ifreq) = etime(clock,t00);
        
        if inteq_savesourcesignals == 1
           filename_sousigs = [Filepath,Filestem,'_f',int2str(ifreq),'_sousigs.mat'];  
           eval(['save ',filename_sousigs,' Qfinal Q_firstterm'])             
        end
        
        
        
        for irec = 1:nreceivers
            t00 = clock;
             IREC = JJ(irec,1:JJnumbofchars(irec));

               Fmatrix = ESIE0calcpropagatematrix(edgedata,edgetoedgedata,edgepairlist,nedgeelems,bigmatrixstartnums,bigmatrixendnums,isthinhole,...
                    Rdata.receivers(irec,:),Rdata.vispartedgesfromr(:,irec),frequency,Rstart,isthinplaneedgepair,...
                    quadraturematrix_pos,quadraturematrix_weights,Rdata.rRsho(Rdata.reftoshortlistR(:,irec)),...
                    Rdata.thetaRsho(Rdata.reftoshortlistR(:,irec)),Rdata.zRsho(Rdata.reftoshortlistR(:,irec)));
             
             P_receiver(ifreq,irec) = Fmatrix*Qfinal;
             tESSIEpropagate(ifreq,irec) = etime(clock,t00);
        end

    elseif doaddsources == 0
        if ifreq==1
            P_receiver = zeros(nfrequencies,nreceivers,nsources);
        end
        for isou = 1:nsources,
            ISOU = JJ(isou,1:JJnumbofchars(isou));

            [Q_firstterm,nedgeelems] = ESIE0integ_souterm(planedata.planeeqs,edgedata,edgetoedgedata,edgepairlist,nedgeelems,bigmatrixstartnums,bigmatrixendnums,...
                inteq_ngauss,inteq_discretizationtype,Sdata.sources(isou,:),...
                Sdata.vispartedgesfroms(:,isou),vispartedgesfromIS,frequency,gaussvectors,Sdata.rSsho(Sdata.reftoshortlistS(:,isou)),...
                Sdata.thetaSsho(Sdata.reftoshortlistS(:,isou)),Sdata.zSsho(Sdata.reftoshortlistS(:,isou)));
                      
            if SHOWTEXT >= 2
                disp('      Starting to iterate, method 5')
            end
            Q = Q_firstterm;
            Qfinal = Q;   
            for iiter = 1:inteq_niter
                if SHOWTEXT >= 3
                   disp(['      Iteration no. ',JJ(iiter,1:JJnumbofchars(iiter)),' of ',JJ(inteq_niter,1:JJnumbofchars(inteq_niter))])
                end
                Q_ackum = Q*0;              
                shortlistprev = 0;
% % %                 Hsub = [];
                 Hsubdata = [];
                ivuse = [];
               for nn = 1:size(listofsubmatrices,1)
                    ivoutvec = bigmatrixstartnums(listofsubmatrices(nn,2)):bigmatrixendnums(listofsubmatrices(nn,2));
                    ivinvec  = bigmatrixstartnums(listofsubmatrices(nn,3)):bigmatrixendnums(listofsubmatrices(nn,3));
                    shortlistnumber = reftoshortlist(nn);
                    if shortlistnumber ~= shortlistprev
% % %                        eval(['Hsub = Hsub',JJ(nn,1:JJnumbofchars(nn)),';']) 
                       eval(['Hsubdata = Hsubdata',int2str(nn),';']) 
                       eval(['ivuse = ivuse',int2str(nn),';']) 
                       nHsub1 = length(ivoutvec);
                       row_Hsub = rem(ivuse-1, nHsub1) + 1;
                       col_Hsub = (ivuse-row_Hsub)/nHsub1 + 1;
                       shortlistprev = shortlistnumber;                   
                    end
% % %                     Q_ackum(ivoutvec) = Q_ackum(ivoutvec) + Hsub*Q(ivinvec);  
                    Q_ackum(1:ivoutvec(end)) = Q_ackum(1:ivoutvec(end)) + accumarray(ivoutvec(1)-1+row_Hsub,Hsubdata.*Q(ivinvec(1)-1+col_Hsub));                      
                end
                Q = Q_ackum; 
                Qfinal = Qfinal + Q; 
            end
            
            if inteq_savesourcesignals == 1
               filename_sousigs = [Filepath,Filestem,'_',ISOU,'_f',int2str(ifreq),'_sousigs.mat'];  
               eval(['save ',filename_sousigs,' Qfinal Q_firstterm'])             
            end
            
            for irec = 1:nreceivers
                 IREC = JJ(irec,1:JJnumbofchars(irec));

                 Fmatrix = ESIE0calcpropagatematrix(edgedata,edgetoedgedata,edgepairlist,nedgeelems,bigmatrixstartnums,bigmatrixendnums,isthinhole,...
                    Rdata.receivers(irec,:),Rdata.vispartedgesfromr(:,irec),frequency,Rstart,isthinplaneedgepair,...
                    quadraturematrix_pos,quadraturematrix_weights,Rdata.rRsho(Rdata.reftoshortlistR(:,irec)),...
                    Rdata.thetaRsho(Rdata.reftoshortlistR(:,irec)),Rdata.zRsho(Rdata.reftoshortlistR(:,irec)));

                 P_receiver(ifreq,irec,isou) = Fmatrix*Qfinal;

            end
        end

    end
end


    