function ESIE0plotmodel(eddatafile,plotoptions,plotoptions2);
% ESIE0plotmodel - Plots a model which is given in an eddatafile.
%
% Input parameters:
%   eddatafile (optional)   The input file. If not specified, a file
%                           opening window will be presented.
%   plotoptions (optional)  An integer that can give extra options:
%                           if bit0 = 1 (= 1) => plot sources
%                           if bit1 = 1 (= 2) => plot Rdata.receivers
%                           if bit2 = 1 (= 4) => plot plane normal vectors
%                           if bit3 = 1 (= 8) => print plane numbers
%                           if bit4 = 1 (=16) => print edge numbers
%                           if bit5 = 1 (=32) => print corner numbers
%                           if bit6 = 1 (=64) => print plane numbers using the
%                                                CAD file numbering
%                           if bit7 = 1 (=128)=> print corner numbers using the
%                                                CAD file numbering
%                           Example: the integer 11 = 1011 binary,
%                           so bits 0,1, and 3 are set.
%   plotoptions2 (optional) A matrix with two columns of 1 or many integers, the first
%                           giving the source numbers to be plotted and the
%                           second giving the receiver numbers to be
%                           plotted. The first in the sequence will be
%                           plotted with a black color.
%
% Sources and Rdata.receivers are taken from an sdatafile and an rdatafile, the file name of
% which is assumed to be similar to the eddatafile.
%
% Uses functions ESIE0strpend, ESIE0myvalueinput
%
% ----------------------------------------------------------------------------------------------
%   This file is part of the Edge Diffraction Toolbox by Peter Svensson.                       
%                                                                                              
%   The Edge Diffraction Toolbox is free software: you can redistribute it and/or modify       
%   it under the terms of the GNU General Public License as published by the Free Software     
%   Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                              
%   The Edge Diffraction Toolbox is distributed in the hope that it will be useful,       
%   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                              
%   You should have received a copy of the GNU General Public License along with the           
%   Edge Diffraction Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
% ----------------------------------------------------------------------------------------------
% Peter Svensson (svensson@iet.ntnu.no) 18 May 2010
%
% ESIE0plotmodel(eddatafile,plotoptions,plotoptions2);

%--------------------------------------------------------------
% Read the eddatafile

if nargin == 0 | (nargin >= 1 & isstr(eddatafile) ~= 1),
	[eddatafile,eddatafilepath] = uigetfile('*eddata.mat','Please select the eddatafile');
    [eddatafilepath,temp1,temp2] = fileparts(eddatafilepath);
	if ~isstr(eddatafile) | isempty(eddatafile),
		return
	end
else
% 	[eddatafilepath,eddatafile] = ESIE0strppath(eddatafile);
	[eddatafilepath,eddatafile,fileext] = fileparts(eddatafile);
    eddatafile = [eddatafile,fileext];
end
Filestem = ESIE0strpend(eddatafile,'_eddata');

disp(['eddatafile is ',eddatafilepath,filesep,eddatafile])

if nargin == 1
    if isstr(eddatafile) ~= 1,
        plotoptions = eddatafile;
    else
        plotoptions = 0;    
    end
elseif nargin == 0,
    plotoptions = 0;    
end

if nargin < 3
    plotoptions2 = [];
else
    nplotoptions2 = size(plotoptions2,1);
end

plotsources = bitget(plotoptions,1);
plotreceivers = bitget(plotoptions,2);
plotnvecs = bitget(plotoptions,3);
plotplnumbers = bitget(plotoptions,4);
plotednumbers = bitget(plotoptions,5);
plotconumbers = bitget(plotoptions,6);
plotplCADnumbers = bitget(plotoptions,7);
plotcoCADnumbers = bitget(plotoptions,8);

if plotplCADnumbers, plotplnumbers = 1; end
if plotcoCADnumbers, plotconumbers = 1; end

%--------------------------------------------------------------------------
% Load the needed input files

if ~isempty(eddatafilepath)
    eval(['load ',eddatafilepath,filesep,eddatafile])
else
    eval(['load ',eddatafile])    
end
    
ncornersperplanevec = double(planedata.ncornersperplanevec);
if plotsources,
    sdatafile = [eddatafilepath,Filestem,'_sdata.mat'];
    if exist(sdatafile) == 2,
        eval(['load ',sdatafile])
    else
        error(['ERROR: The sdata file named ',sdatafile,' could not be opened'])    
    end
end
if plotreceivers,
    rdatafile = [eddatafilepath,Filestem,'_rdata.mat'];
    if exist(rdatafile) == 2,
        eval(['load ',rdatafile])
    else
        error(['ERROR: The rdata file named ',rdatafile,' could not be opened'])    
    end
end

if plotplCADnumbers | plotcoCADnumbers,
    cadgeofile = [eddatafilepath,Filestem,'_cadgeo.mat'];
    if exist(cadgeofile) == 2,
        eval(['load ',cadgeofile])
    else
        error(['ERROR: The cadgeo file named ',cadgeofile,' could not be opened'])            
    end
end
%--------------------------------------------------------------

ncorners = size(planedata.corners,1);
nedges = size(edgedata.edgecorners,1);
nplanes = size(planedata.planeeqs,1);

planelist = 1:nplanes;

viewpos = ESIE0myvalueinput('From which point do you want to watch the model? (x y z, or az el, with spaces inbetween) ',-1);

%plot3(Rdata.receivers(:,1),Rdata.receivers(:,2),Rdata.receivers(:,3),'o');

% figure(1)
hold off
for ii = 1:nedges,
	co1 = edgedata.edgecorners(ii,1);
	co2 = edgedata.edgecorners(ii,2);
	iv = [co1;co2];
	h = plot3(planedata.corners(iv,1),planedata.corners(iv,2),planedata.corners(iv,3));
    set(h,'LineWidth',2);
    if ii ==1,
        view(viewpos)
        hold
    end
end

if plotnvecs,
	for ii = 1:nplanes,
        midpoint = mean(planedata.corners(planedata.planecorners(ii,1:(planedata.ncornersperplanevec(ii))),:));
        
        endpoint = midpoint + planedata.planeeqs(ii,1:3);
        bothpoints = [midpoint;endpoint];
        plot3(bothpoints(:,1),bothpoints(:,2),bothpoints(:,3));
        plot3(midpoint(1),midpoint(2),midpoint(3),'ro');
	end
end

if plotplnumbers,
	for ii = 1:nplanes,
        midpoint = mean(planedata.corners(planedata.planecorners(ii,1:planedata.ncornersperplanevec(ii)),:));
        endpoint = midpoint + planedata.planeeqs(ii,1:3)*0.1;
        if plotplCADnumbers == 1,
            h = text(endpoint(1),endpoint(2),endpoint(3),['p',int2str(planenumbers(ii))]);
            set(h,'FontSize',14);
        else
            h = text(endpoint(1),endpoint(2),endpoint(3),['p',int2str(ii)]);
            set(h,'FontSize',14);
        end
    
	end
end

if plotednumbers,
    for ii = 1:nedges,
        midpoint = mean(corners(edgecorners(ii,1:2),:));
        h = text(midpoint(1),midpoint(2),midpoint(3),['e',int2str(ii)]);
            set(h,'FontSize',18);
    end
end

if plotconumbers,
    for ii = 1:ncorners,
        if plotcoCADnumbers == 1,
            h = text(corners(ii,1),corners(ii,2),corners(ii,3),['c',int2str(cornernumbers(ii))]); 
            set(h,'FontSize',18);
        else
            h = text(corners(ii,1),corners(ii,2),corners(ii,3),['c',int2str(ii)]);    
            set(h,'FontSize',18);
        end
    end
    
end

if plotsources == 1,
    if isempty(plotoptions2)
        plot3(Sdata.sources(:,1),Sdata.sources(:,2),Sdata.sources(:,3),'*');
    else
        if nplotoptions2 == 1
            plot3(sources(plotoptions2(1),1),sources(plotoptions2(1),2),sources(plotoptions2(1),3),'*')
        else
            plot3(sources(plotoptions2(1,1),1),sources(plotoptions2(1,1),2),sources(plotoptions2(1,1),3),'k*')
            plot3(sources(plotoptions2(2:nplotoptions2,1),1),sources(plotoptions2(2:nplotoptions2,1),2),sources(plotoptions2(2:nplotoptions2,1),3),'*')            
        end
    end
end



if plotreceivers == 1,
    if isempty(plotoptions2)
        plot3(Rdata.receivers(:,1),Rdata.receivers(:,2),Rdata.receivers(:,3),'ro');
    else
        if nplotoptions2 == 1
            plot3(Rdata.receivers(plotoptions2(2),1),Rdata.receivers(plotoptions2(2),2),Rdata.receivers(plotoptions2(2),3),'ro');
        else
            plot3(Rdata.receivers(plotoptions2(1,2),1),Rdata.receivers(plotoptions2(1,2),2),Rdata.receivers(plotoptions2(1,2),3),'ko');
            plot3(Rdata.receivers(plotoptions2(2:nplotoptions2,2),1),Rdata.receivers(plotoptions2(2:nplotoptions2,2),2),Rdata.receivers(plotoptions2(2:nplotoptions2,2),3),'ro');
            
        end
    end
end
